#include "twComm.h"


// global variable to store data recieved by the authThread
struct sockaddr_in authClientInfo;	// stores data about the user that has requested to join the server
extern int twUpdateUser(int userID,twProtocol *data);
extern int twSendACK(struct sockaddr_in*, int userID);
int authNewClient = 0;		// flag set by authThread checked by process to acknowledge new client. 0 = nothing, 1 = read struct
int clientData = 0;		// flag set by recvThread checked by process to acknowledge new client. 0 = nothing, 1 = read struct

int recvData = 0;		// flag to acknowledge there is recv data to process
pthread_mutex_t authMutex;
pthread_mutex_t recvMutex;	// mutex to control access to authThread and recvThread shared variables
string serverIP;

int outSHandle = 0;
/* Sets up 3 socket handles - 
AUTHPORT-9779 for handling incoming add requests
INPORT-9780 for handling incoming data from clients
OUTPORT-9781 for sending data to clients
Returns -1 if it fails
*/

int twSetupSockets(string compName)
{

// handle variables declared as static so that their values are not destroyed when this function ends 
static int outSHANDLE = 0;
static int inSHANDLE = 0;
static int authSHANDLE = 0;				// For holding the file descriptors of the sockets created 
struct hostent *server;					// for storing details about the server machine
struct sockaddr_in lSOCK; 				// local socket info 
struct in_addr h_addr; 					// host is a internet address structure

memset(&h_addr, 0, sizeof(h_addr)); 


// DNS resolving of local PC
if ((server = gethostbyname(compName.c_str())) == NULL)
	{
		cout << "\nerror resolving DNS\n";
		return -1;
	}

cout << "Server: " << compName << " resolved to IP: " << inet_ntoa(*(struct in_addr*)server->h_addr) << endl;


	
// Socket Setup UDP
/*----------------------------------------------------------------------------------------------*/
if ((inSHANDLE = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) //  making a socket and using the IP protocol(AF_INET) and UDP(SOCK_STREAM), storing its number in inSHANDLE 
	{
		cout << "Error: cannot get socket (server)\n";		//  error checking - socket function will return less than 0 if it fails 
		return -1;
	}
cout << "inSHANDLE = " << inSHANDLE << endl;
if ((outSHANDLE = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) //  making a socket and using the IP protocol(AF_INET) and UDP(SOCK_STREAM), storing its number in outSHANDLE 
	{
		cout << "Error: cannot get socket (server)\n";		//  error checking - socket function will return less than 0 if it fails 
		return -1;
	}
cout << "outSHANDLE = " << outSHANDLE << endl;
if ((authSHANDLE = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) //  making a socket and using the IP protocol(AF_INET) and UDP(SOCK_STREAM), storing its number in authSHANDLE 
	{
		cout << "Error: cannot get socket (server)\n";		//  error checking - socket function will return less than 0 if it fails 
		return -1;
	}
cout << "authSHANDLE = " << authSHANDLE << endl;
cout << "\nInitial Socket Setup\n";


// setting up our local server information for the authport
memset(&lSOCK, 0, sizeof(lSOCK)); 	// sets all the memory values of lSOCK to be 0 
lSOCK.sin_family = AF_INET; 		// says that the socket will use the IP protocol (?) 
lSOCK.sin_port = htons(AUTHPORT);	// setting the local port for the socket to use 
lSOCK.sin_addr.s_addr = inet_addr(inet_ntoa(*(struct in_addr*)server->h_addr));	// address of the local computer, used for binding the socket to the local network adapter 

/*
serverIP = inet_ntoa(*(struct in_addr*)server->h_addr);*/


// binding the AUTH socket to the hardware
if (bind(authSHANDLE,(struct sockaddr *) &lSOCK, sizeof(struct sockaddr_in)) < 0)
	{
		cout << "ERROR, YARRR I CANNY BIND THE SOCKET AUTH (server)";
		return -1; 
	}
cout << "bound authSHANDLE = " << authSHANDLE << endl;

// binding the IN socket to the hardware
lSOCK.sin_port = htons(INPORT);
if (bind(inSHANDLE,(struct sockaddr *) &lSOCK, sizeof(struct sockaddr_in)) < 0)
	{
		cout << "ERROR, YARRR I CANNY BIND THE SOCKET IN (server)";
		return -1; 
	}

cout << "bound inSHANDLE = " << inSHANDLE << endl;

// binding the OUT socket to the hardware
lSOCK.sin_port = htons(OUTPORT);
if (bind(outSHANDLE,(struct sockaddr *) &lSOCK, sizeof(struct sockaddr_in)) < 0)
	{
		cout << "ERROR, YARRR I CANNY BIND THE SOCKET OUT (server)";
		return -1; 
	}

outSHandle = outSHANDLE;

cout << "bound outSHANDLE = " << outSHANDLE << endl;

cout << "\nServer started on IP: " << inet_ntoa(*(struct in_addr*)server->h_addr) << endl;
cout << "\nListening for connections on port " << AUTHPORT << "\nSending Data on port " << OUTPORT << "\nReceiving data on port " << INPORT << endl;



// Setup threads and mutexes

pthread_t authThread, recvThread;	 // this is the thread handle for the auth thread
pthread_attr_t threadAttr;		// stores info to set thread to be detached (not need to be rejoined to main process after terminating
int ret;

ret = pthread_mutex_init(&authMutex, NULL);
if(ret!=0)
	{
	perror("authMutex init failed \n");
	exit(EXIT_FAILURE);
	}

ret = pthread_mutex_init(&recvMutex, NULL);
if(ret!=0)
	{
	perror("recvMutex init failed \n");
	exit(EXIT_FAILURE);
	}
// setup thread attribute to be detatched
ret = pthread_attr_init(&threadAttr);
if(ret!=0)
	{
	perror("threadAttr init failed \n");
	exit(EXIT_FAILURE);
	}
ret = pthread_attr_setdetachstate(&threadAttr, PTHREAD_CREATE_DETACHED);
if (ret!=0)
	{
	perror("threadAttr set detatch failed \n");
	exit(EXIT_FAILURE);
	}
// create threads
ret = pthread_create(&authThread,&threadAttr, twRunAuthThread, (void*)&authSHANDLE); // create a thread that runs the auth thread function
if(ret!=0)
	{
	perror("authThread creation failed \n");
	exit(EXIT_FAILURE);
	}
ret = pthread_create(&recvThread,&threadAttr, twRunRecvThread, (void*)&inSHANDLE); // create a thread that runs the auth thread function
if(ret!=0)
	{
	perror("recvThread creation failed \n");
	exit(EXIT_FAILURE);
	}

// destroy unused pthread attr 
(void)pthread_attr_destroy(&threadAttr);
// reset value in global variable authClientInfo
memset(&authClientInfo, 0, sizeof(authClientInfo));


return 0;
}



// Sets up and runs thread listening for incoming connections
void *twRunAuthThread(void* arg)
{

int * temp = (int*) arg;
int authSHANDLE = *temp;
cout << "authSHANDLE inside authThread =  " << authSHANDLE << endl;

	int rVal = 0;
	char buffer[MSGSIZE] = {0};
	struct sockaddr_in dsClient; // structure to store dgram origin address
	socklen_t dsLen;
	
	dsLen = sizeof(dsClient);

	// wait to recieve authentication datagrams
	cout << "authThread - Awaiting Connections.. \n" << endl;
	
	// main while loop. This waits for packets from DS, stores the IP of the sender, and then flags the main process.
	while(1)
	{
	// reset client host structure and message buffer
	memset(&dsClient, 0, sizeof(dsClient));
	memset(&buffer, 0 , sizeof(buffer));

	if ((rVal = recvfrom(authSHANDLE, buffer, MSGSIZE, 0, (struct sockaddr *) &dsClient, &dsLen)) < 0)
		{
		perror("authThread - error receiving from client \n");
		exit(EXIT_FAILURE);
		}
	
	// process buffer sent from client here.
	else
		{
		cout << "authThread - Recieved: " << buffer << " : from: " << inet_ntoa(dsClient.sin_addr) << endl;
		if(!strcmp(buffer, "ADDME"))
			cout << "authThread - client wants to be added to recv list" << endl;
		// mark flag as new client to process. lock mutex to prevent process accessing shared variable. if already locked it will block
		// until available
		pthread_mutex_lock(&authMutex);
		// register new client to add
		authNewClient = 1;	
		// copy client host info to global variable for process to deal with 
		memcpy((void*)&authClientInfo, (void*)&dsClient, sizeof(sockaddr_in));
		
		// unlock variable
		pthread_mutex_unlock(&authMutex);
		}

	
	
	}

}


// Sets up and runs thread for receiving data from game clients
void *twRunRecvThread(void* arg)
{
int *temp = (int*) arg;
int recvSHANDLE = *temp;
cout << "recvSHANDLE inside recvThread =  " << recvSHANDLE << endl;

	int rVal = 0;
	twProtocol recvBuffer; 
	struct sockaddr_in dsClient; // structure to store dgram origin address
	socklen_t dsLen;
	
	dsLen = sizeof(dsClient);

	// wait to recieve authentication datagrams
	cout << "recvThread - Awaiting Data.. \n" << endl;
	
	// main while loop. This waits for packets from DS, stores the IP of the sender, and then flags the main process.
	while(1)
	{
	// reset client host structure and message buffer
	memset(&dsClient, 0, sizeof(dsClient));
	memset(&recvBuffer, 0 , sizeof(recvBuffer));

	if ((rVal = recvfrom(recvSHANDLE, &recvBuffer, sizeof(twProtocol), 0, (struct sockaddr *) &dsClient, &dsLen)) < 0)
	{
		perror("recvThread - error receiving from client \n");
		exit(EXIT_FAILURE);
	}
	
	// process buffer sent from client here. do not go via main process otherwise packets might be overwritten before dealt with
	else
	{
		cout << "recvThread - Recieved: " << recvBuffer.X << "," << recvBuffer.Y << " ID:" << recvBuffer.userID << " from: " << inet_ntoa(dsClient.sin_addr) << endl;
		
		
		// until available
		pthread_mutex_lock(&recvMutex);
		// update client database
		if((twUpdateUser(recvBuffer.userID,&recvBuffer)) < 0)
		{
			cout << "Error updating client ID: " << recvBuffer.userID << endl;
		}
		// unlock variable
		pthread_mutex_unlock(&recvMutex);
	}
	}

return 0;
}

/// This function sends the DS an acknowledgement packet including its userID for future packets
int twSendACK(struct sockaddr_in* target, int userID) // returns -1 if fail
{


int sentBytes = 0;
const struct sockaddr_in temp = *target;

if((sentBytes = sendto(outSHandle, &userID, sizeof(userID), 0, (struct sockaddr*)&temp,sizeof(temp))) != sizeof(userID))
{
	cout << "Error sending ACK packet to DS IP: " << inet_ntoa(target->sin_addr) << endl;
	return -1;
}

cout << "Sent ACK Packet to DS IP: " << inet_ntoa(target->sin_addr) << " userID: " << userID << endl;

return 0;
}

