#include "twMainHeader.h"



/* function to release/destroy our resources and restoring the old desktop */
void Quit( int returnCode )
{
    /* clean up the window */
    SDL_Quit( );

    /* and exit appropriately */
    exit( returnCode );
}

/* function to reset our viewport after a window resize */
int resizeWindow( int width, int height )
{
    /* Height / width ration */
    GLfloat ratio;

    /* Protect against a divide by zero */
    if ( height == 0 )
	height = 1;

    ratio = ( GLfloat )width / ( GLfloat )height;

    /* Setup our viewport. */
    glViewport( 0, 0, ( GLsizei )width, ( GLsizei )height );

    /* change to the projection matrix and set our viewing volume. */
    glMatrixMode( GL_PROJECTION );
    glLoadIdentity( );

    /* Set our perspective */


    gluPerspective( 138.0f, ratio, 0.1f, 100.0f );


    /* Make sure we're chaning the model view and not the projection */
    glMatrixMode( GL_MODELVIEW );

    /* Reset The View */
    glLoadIdentity( );

    return( TRUE );
}

/* function to handle key press events */
void handleKeyPress( SDL_keysym *keysym )
{
    switch ( keysym->sym )
	{
	case SDLK_ESCAPE:
	    /* ESC key was pressed */
	    Quit( 0 );
	    break;
	case SDLK_F1:
	    /* F1 key was pressed
	     * this toggles fullscreen mode
	     */
	    SDL_WM_ToggleFullScreen( surface );
	    break;
	default:
	    break;
	}

    return;
}

/* general OpenGL initialization function */
int initGL( GLvoid )
{

    /* Enable smooth shading */
    glShadeModel( GL_SMOOTH );

    /* Set the background black */
    glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );

    /* Depth buffer setup */
    glClearDepth( 1.0f );

    /* Enables Depth Testing */
    glEnable( GL_DEPTH_TEST );

    /* The Type Of Depth Test To Do */
    glDepthFunc( GL_LEQUAL );

    /* Really Nice Perspective Calculations */
    glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );

    return( TRUE );
}

/* Here goes our drawing code */
int drawGLScene( GLvoid )
{
    /* Clear The Screen And The Depth Buffer */
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    glLoadIdentity();
    
	int userID;
	int X,Y;
	unsigned int i;
	
	if(twUserDB.size() != 0)
	// iterate through users and find user to update
	for(i=0;i<twUserDB.size();i++)
	{
		glLoadIdentity();
		X = twUserDB[i]->getX();
		Y = twUserDB[i]->getY();
		userID = twUserDB[i]->getID();
		//translate to origin
			

		// translate to player locations
		glTranslatef((X*2)-256, (Y*2)-256, -100.0);
		
			
		// draw box representing player
		glColor3f(1.0/userID+1, 1.0/userID+1, 1.0/userID+10);
		glScalef(10.0,10.0,10.0);
		glBegin(GL_LINES);
		glVertex3f(0.5,0.5,0);
		glVertex3f(-0.5,-0.5,0);
		glVertex3f(0.5,-0.5,0);
		glVertex3f(-0.5,0.5,0);
		glVertex3f(0.5,0.5,0);
		glEnd();
	}

			
	

   
    /* Draw it to the screen */
    SDL_GL_SwapBuffers( );


    return(TRUE);
}

int openGLinit()
{
    /* Flags to pass to SDL_SetVideoMode */
    int videoFlags;
    /* main loop variable */
    int done = FALSE;
    /* used to collect events */
    SDL_Event event;
    /* this holds some info about our display */
    const SDL_VideoInfo *videoInfo;
    /* whether or not the window is active */
    int isActive = TRUE;

    /* initialize SDL */
    if ( SDL_Init( SDL_INIT_VIDEO ) < 0 )
	{
	    fprintf( stderr, "Video initialization failed: %s\n",
		     SDL_GetError( ) );
	    Quit( 1 );
	}

    /* Fetch the video info */
    videoInfo = SDL_GetVideoInfo( );

    if ( !videoInfo )
	{
	    fprintf( stderr, "Video query failed: %s\n",
		     SDL_GetError( ) );
	    Quit( 1 );
	}

    /* the flags to pass to SDL_SetVideoMode */
    videoFlags  = SDL_OPENGL;          /* Enable OpenGL in SDL */
    videoFlags |= SDL_GL_DOUBLEBUFFER; /* Enable double buffering */
    videoFlags |= SDL_HWPALETTE;       /* Store the palette in hardware */
    videoFlags |= SDL_RESIZABLE;       /* Enable window resizing */

    /* This checks to see if surfaces can be stored in memory */
    if ( videoInfo->hw_available )
	videoFlags |= SDL_HWSURFACE;
    else
	videoFlags |= SDL_SWSURFACE;

    /* This checks if hardware blits can be done */
    if ( videoInfo->blit_hw )
	videoFlags |= SDL_HWACCEL;

    /* Sets up OpenGL double buffering */
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

    /* get a SDL surface */
    surface = SDL_SetVideoMode( SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP,
				videoFlags );

    /* Verify there is a surface */
    if ( !surface )
	{
	    fprintf( stderr,  "Video mode set failed: %s\n", SDL_GetError( ) );
	    Quit( 1 );
	}

    /* initialize OpenGL */
    initGL( );

    /* resize the initial window */
    resizeWindow( SCREEN_WIDTH, SCREEN_HEIGHT );

    /* wait for events */
    while ( !done )
	{
	    /* handle the events in the queue */

	    while ( SDL_PollEvent( &event ) )
		{
		    switch( event.type )
			{
			case SDL_ACTIVEEVENT:
			    /* Something's happend with our focus
			     * If we lost focus or we are iconified, we
			     * shouldn't draw the screen
			     */
			    if ( event.active.gain == 0 )
				isActive = FALSE;
			    else
				isActive = TRUE;
			    break;			    
			case SDL_VIDEORESIZE:
			    /* handle resize event */
			    surface = SDL_SetVideoMode( event.resize.w,
							event.resize.h,
							16, videoFlags );
			    if ( !surface )
				{
				    fprintf( stderr, "Could not get a surface after resize: %s\n", SDL_GetError( ) );
				    Quit( 1 );
				}
			    resizeWindow( event.resize.w, event.resize.h );
			    break;
			case SDL_KEYDOWN:
			    /* handle key presses */
			    handleKeyPress( &event.key.keysym );
			    break;
			case SDL_QUIT:
			    /* handle quit requests */
			    done = TRUE;
			    break;
			default:
			    break;
			}
		}

		
	    /* draw the scene */
	    twHandleClients();
	    drawGLScene( );
	}

    /* clean ourselves up and exit */
    Quit( 0 );

    /* Should never get here */
    return( 0 );
}



int main(int argc, char * argv[])
{

//check for computer name as first argument
if(argc!=2)
{
	cout << "Please specify the name of the machine to connect the server to \n";
	exit(EXIT_FAILURE);
}

if(twSetupSockets(argv[1]) < 0)
{
	cout << "Error Socket Setup\n";
	exit(EXIT_FAILURE);
}


// Run main while loop
openGLinit();

return 0;
}


void twHandleClients(void)
{
//Check for new connections from authThread
	//sleep(2); // THIS IS TEMPORARY!

	// handling new clients
	pthread_mutex_lock(&authMutex);
	if(authNewClient == 1)
	{
		
		cout << "main process authClientInfo: " << inet_ntoa(authClientInfo.sin_addr) << endl;
		authNewClient = 0; // reset flag
		int ipCheck = twCheckClientIP(&authClientInfo);// check to see if new client is already in database. returns 1 if not in database

		if(ipCheck == 1)
		{
			int ID = twAddUser(&authClientInfo); //add user to database - careful not to share main variable!!!
			cout << "Added user IP: "<< inet_ntoa(authClientInfo.sin_addr) << " to database" << endl;
			// send ACK packet to give DS its userID
			twSendACK(&authClientInfo, ID);
			cout << "Sent ACK packet to IP: " << inet_ntoa(authClientInfo.sin_addr) << endl;
		}

		else if(ipCheck == 0)
			cout << "Client IP: " << inet_ntoa(authClientInfo.sin_addr) << " already in database. Not adding client" << endl;
	
		
	}
	pthread_mutex_unlock(&authMutex);
	//Manage received data from recvThread TODO
	
// 	twDisplayTotalUsers();
	twDisplayAllUsers();
	


	//Send data to clients TODO
}




